-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_ManipulateBones"

-- **************************************************
-- General information about this script
-- **************************************************

DS_ManipulateBones = {}


function DS_ManipulateBones:Name()
	return "Manipulate Bones"
end

function DS_ManipulateBones:Version()
	return "1.0"
end

function DS_ManipulateBones:Description()
	return "Manipulate bones. Hold SHIFT to scale, CTRL to rotate a single bone or ALT to translate bone."
end

function DS_ManipulateBones:Creator()
	return "dauid.com"
end

function DS_ManipulateBones:UILabel()
	return "Manipulate Bones"
end

-- **************************************************
-- Recurring values
-- **************************************************

DS_ManipulateBones.selID = -1
DS_ManipulateBones.lastVec = LM.Vector2:new_local()
DS_ManipulateBones.keyEveryMode = false

-- **************************************************
-- The guts of this script
-- **************************************************


function DS_ManipulateBones:OnMouseDown(moho, mouseEvent, keyEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end


	
	if mouseEvent.shiftKey then
		LM_ScaleBone:OnMouseDown(moho, mouseEvent)
		return
	elseif mouseEvent.altKey then
		LM_TranslateBone:OnMouseDown(moho, mouseEvent)
		return
	elseif mouseEvent.ctrlKey then
		LM_RotateBone:OnMouseDown(moho, mouseEvent)
		return
	else
	LM_ManipulateBones:OnMouseDown(moho, mouseEvent)
	return
	end
	
end

function DS_ManipulateBones:OnMouseMoved(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end


	
	if mouseEvent.shiftKey then
		LM_ScaleBone:OnMouseMoved(moho, mouseEvent)
		return
	elseif mouseEvent.altKey then
		LM_TranslateBone:OnMouseMoved(moho, mouseEvent)
		return
	elseif mouseEvent.ctrlKey then
		LM_RotateBone:OnMouseMoved(moho, mouseEvent)
		return
	else
	LM_ManipulateBones:OnMouseMoved(moho, mouseEvent)
		return
	end
end

function DS_ManipulateBones:OnMouseUp(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if (self.keyEveryMode) then
	DS_ManipulateBones:keyEveryBone(moho)
	end
	
	if mouseEvent.shiftKey then
		LM_ScaleBone:OnMouseUp(moho, mouseEvent)
		return
	elseif mouseEvent.altKey then
		LM_TranslateBone:OnMouseUp(moho, mouseEvent)
		return
	elseif mouseEvent.ctrlKey then
		LM_RotateBone:OnMouseUp(moho, mouseEvent)
		return
	else
		LM_ManipulateBones:OnMouseUp(moho, mouseEvent)
		return
	end
	

		
end

function DS_ManipulateBones:keyEveryBone(moho)
local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end
	for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			bone.fAnimPos:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE_T)
			bone.fAnimAngle:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE)
			bone.fAnimScale:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE_S)
			moho.layer:UpdateCurFrame()
			moho:UpdateBonePointSelection()	
		end
end

---------------------------------------------------------------------

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************


DS_ManipulateBones.FREEZE = MOHO.MSG_BASE
DS_ManipulateBones.TRANSLATION = MOHO.MSG_BASE+1
DS_ManipulateBones.SCALE = MOHO.MSG_BASE +2
DS_ManipulateBones.ROTATION = MOHO.MSG_BASE +3
DS_ManipulateBones.LOCK = MOHO.MSG_BASE +4
DS_ManipulateBones.RESET = MOHO.MSG_BASE +5
DS_ManipulateBones.KEYALL = MOHO.MSG_BASE +6
DS_ManipulateBones.HELP = MOHO.MSG_BASE +7
DS_ManipulateBones.KEYEVERY = MOHO.MSG_BASE +8

function DS_ManipulateBones:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.Button("Freeze pose", self.FREEZE))
	layout:AddChild(LM.GUI.Button("Key all", self.KEYALL))
	layout:AddChild(LM.GUI.Button("Reset all", self.RESET))
	layout:AddChild(LM.GUI.StaticText("Reset selected:"))
	layout:AddChild(LM.GUI.Button("Translation", self.TRANSLATION))
	layout:AddChild(LM.GUI.Button("Scale", self.SCALE))
	layout:AddChild(LM.GUI.Button("Rotation", self.ROTATION))
	self.lockBone = LM.GUI.CheckBox("Lock bone", self.LOCK)
	layout:AddChild(LM.GUI.Divider(true))
	layout:AddChild(self.lockBone)
	layout:AddChild(LM.GUI.Divider(true))
	self.keyEvery = LM.GUI.CheckBox("Key all with every edit", self.KEYEVERY)
	layout:AddChild(self.keyEvery)
	self.keyEvery:SetValue(self.keyEveryMode)
	layout:AddChild(LM.GUI.Divider(true))
	layout:AddChild(LM.GUI.Button("?", self.HELP))
	

end

function DS_ManipulateBones:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
			return
	end

	local selID = skel:SelectedBoneID()
	local bone = skel:Bone(selID)
	self.lockBone:SetValue(bone.fIKLock.value)
end


function DS_ManipulateBones:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end
	
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
------------------------------------------------------------------Freeze pose clicked
	if (msg == self.FREEZE) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fAnimPos:CountKeys()>1) then
			bone.fAnimPos:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE_T)
			end
			if (bone.fAnimAngle:CountKeys()>1) then
			bone.fAnimAngle:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE)
			end
			if (bone.fAnimScale:CountKeys()>1) then
			bone.fAnimScale:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE_S)
			end
			moho.layer:UpdateCurFrame()
			moho:UpdateBonePointSelection()
			
			
			
		end
	end
	------------------------------------------------------------------Key all clicked
	if (msg == self.KEYALL) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			bone.fAnimPos:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE_T)
			bone.fAnimAngle:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE)
			bone.fAnimScale:AddKey( moho.frame )
			moho:NewKeyframe(CHANNEL_BONE_S)
			moho.layer:UpdateCurFrame()
			moho:UpdateBonePointSelection()	
		end
	end
------------------------------------------------------------------Reset rotation clicked
	if (msg == self.ROTATION) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					bone.fAnimAngle:SetValue(moho.layerFrame, bone.fAnimAngle:GetValue(0))
				end
			end
			moho.layer:UpdateCurFrame()
			moho:NewKeyframe(CHANNEL_BONE)
		end
	end
------------------------------------------------------------------Reset scale clicked
	if (msg == self.SCALE) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					bone.fAnimScale:SetValue(moho.layerFrame, bone.fAnimScale:GetValue(0))
				end
			end
			moho.layer:UpdateCurFrame()
			moho:NewKeyframe(CHANNEL_BONE_S)

		end
	end
------------------------------------------------------------------Reset translation clicked
	if (msg == self.TRANSLATION) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				moho.document:PrepUndo(moho.layer)
				moho.document:SetDirty()
				bone.fAnimPos:SetValue(moho.layerFrame, bone.fAnimPos:GetValue(0))
				moho.layer:UpdateCurFrame()
				moho:NewKeyframe(CHANNEL_BONE_T)
				break
			end
		end
	end
------------------------------------------------------------------Lock bone clicked
	if (msg == self.LOCK) then	
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					if (self.lockBone:Value()) then
						skel:LockBone(i, moho.layerFrame)
					else
						skel:UnlockBone(i, moho.layerFrame)
					end
				end
			end
			moho.layer:UpdateCurFrame(true)
			moho:NewKeyframe(CHANNEL_BONE)
			moho:NewKeyframe(CHANNEL_BONE_LOCK)
		end
	end
------------------------------------------------------------------Reset all clicked
	if (msg == self.RESET) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			
			if (bone.fAnimPos:CountKeys()>1) then
			bone.fAnimPos:SetValue(moho.layerFrame, bone.fAnimPos:GetValue(0))
			moho:NewKeyframe(CHANNEL_BONE_T)
			end
			if (bone.fAnimAngle:CountKeys()>1) then
			bone.fAnimAngle:SetValue(moho.layerFrame, bone.fAnimAngle:GetValue(0))
			moho:NewKeyframe(CHANNEL_BONE)
			end
			if (bone.fAnimScale:CountKeys()>1) then
			bone.fAnimScale:SetValue(moho.layerFrame, bone.fAnimScale:GetValue(0))
			moho:NewKeyframe(CHANNEL_BONE_S)
			end
			moho.layer:UpdateCurFrame()
		
		end
	end
------------------------------------------------------------------Help clicked
	if (msg == self.HELP) then
		LM.GUI.Alert(LM.GUI.ALERT_INFO, "Freeze pose keys only previously keyed properties.", "For example, if you've only rotated bones, only rotation will be keyed.", "Also, this tool will not work on frame 0. Not sure why.", "OK", nil, nil)
	end
------------------------------------------------------------------Key every mode
	if (msg == self.KEYEVERY) then
		self.keyEveryMode=self.keyEvery:Value()	
	end
	
end
